<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class ParceiroNegocioFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection       = $entityManager->getConnection();
    }
    
    public function replicaInteprosp($paramsInsertInterlocutor){
        
        $insertInterlocutor = "INSERT INTO INTEPROSP (CDPROSPECT, CDINTEPROS, IDTPINTEPROS, NMINTEPROS, DSEMAILIPROS, NRTEL1INPROS, NRORG, DSCARACINPROS, IDINPROSRECPRES)
                               VALUES(:CDPROSPECT,
                                      :CDINTEPROS,
                                      'O',
                                      :NMINTEPROS,
                                      :DSEMAILIPROS,
                                      :NRTEL1INPROS,
                                      :NRORG,
                                      :DSCARACINPROS,
                                      'S')";
        
        try{
            $std = $this->connection->prepare($insertInterlocutor);
            $std->execute($paramsInsertInterlocutor);
            $paramsSelectNewParc = array(
                    "CDPROSPECT" => $paramsInsertInterlocutor['CDPROSPECT'],
                    "CDINTEPROS" => $paramsInsertInterlocutor['CDINTEPROS'],
                    "NRORG" => $paramsInsertInterlocutor['NRORG']
                );
            $newparc = $this->getNewParcInteprosp($paramsSelectNewParc);
            return $newparc;
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function updateInteprosp($paramsInsertInterlocutor){

        $updatetInterlocutor = "UPDATE INTEPROSP SET 
                            NMINTEPROS = :NMINTEPROS, DSEMAILIPROS = :DSEMAILIPROS, NRTEL1INPROS = :NRTEL1INPROS, DSCARACINPROS = :DSCARACINPROS WHERE CDPROSPECT = :CDPROSPECT AND CDINTEPROS = :CDINTEPROS AND NRORG = :NRORG";
        
        try{
            $std = $this->connection->prepare($updatetInterlocutor);
            $std->execute($paramsInsertInterlocutor);
            $paramsSelectNewParc = array(
                    "CDPROSPECT" => $paramsInsertInterlocutor['CDPROSPECT'],
                    "CDINTEPROS" => $paramsInsertInterlocutor['CDINTEPROS'],
                    "NRORG" => $paramsInsertInterlocutor['NRORG']
                );
            $newparc = $this->getNewParcInteprosp($paramsSelectNewParc);
            return $newparc;
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function getCdintepros($params){
        
        $selectInterlocutor = " SELECT CDINTEPROS FROM INTEPROSP WHERE NRRELACIONAPARC = :NRRELACIONAPARC and NRORG = :NRORG ";
        
        try{
            $std = $this->connection->prepare($selectInterlocutor);
            $std->execute();
            $cdIntepros = $std->fetchAll();
            return $cdIntepros[0]['CDINTEPROS'];
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function getNewParcInteprosp($params){
        
        $select = "SELECT PN.NRPARCNEGOCIO, RP.NRRELACIONAPARC FROM
                    PARCNEGOCIO PN
                    LEFT JOIN RELACIONAPARC RP
                    ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                    AND RP.NRORG = PN.NRORG
                    LEFT JOIN INTEPROSP IP
                    ON IP.NRRELACIONAPARC = RP.NRRELACIONAPARC
                    AND IP.NRORG = RP.NRORG
                    WHERE IP.CDPROSPECT = :CDPROSPECT
                    AND IP.CDINTEPROS = :CDINTEPROS
                    AND IP.NRORG = :NRORG";
        try{
            $std = $this->connection->prepare($select);
            $std->execute($params);
            $newparc = $std->fetchAll();
            return $newparc[0];
        }
        catch(\Exception $e){
            throw $e;
        };
    }
    
    public function deleteInteprosp($params){
        
        $deleteInterlocutor = "DELETE FROM INTEPROSP WHERE NRRELACIONAPARC = :NRRELACIONAPARC and NRORG = :NRORG ";
        
        try{
            $std = $this->connection->prepare($deleteInterlocutor);
            $std->execute();
        }
        catch(\Exception $e){
            throw $e;
        };
        
    }
    
    public function atualizarCaminhoLogo(array $params) {
        $sql = "UPDATE PARCNEGOCIO
                   SET DSDIRLOGOMARCA = :DSDIRLOGOMARCA
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function getEnderecoPrincipalLocalAtendimento(array $params) {
        $sql = "SELECT ENDE.CDPAIS,
                       ENDE.SGESTADO,
                       ENDE.CDMUNICIPIO
                  FROM ENDERECOPARC ENDE
                  JOIN ESTRUTURAM EST
                    ON ENDE.NRPARCNEGOCIO = EST.NRPARCNEGOCIO
                   AND ENDE.NRORG         = EST.NRORG
                 WHERE EST.NRESTRUTURAM    = :NRESTRUTURAM
                   AND ENDE.NRORG          = :NRORG
                   AND ENDE.CDTIPOENDERECO = 'PRINCIPAL'
                   AND ENDE.IDATIVO        = 'S'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getEnderecoPrincipalParceiro(array $params) {
        $sql = "SELECT CDPAIS,
                       SGESTADO,
                       CDMUNICIPIO
                  FROM ENDERECOPARC
                 WHERE NRPARCNEGOCIO  = :NRPARCNEGOCIO
                   AND NRORG          = :NRORG
                   AND CDTIPOENDERECO = 'PRINCIPAL'
                   AND IDATIVO        = 'S'";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
}