<?php
namespace Teknisa\SM\Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Factory\Sm00001000ItemFormPrecoPropostaFactory;
use Teknisa\SM\Factory\Sm00001000ItemLancamentoPropostaFactory;
use Teknisa\SM\Util\Procedure\FctCopiaFormPrecoItprop;

class Sm00001000ItemFormPrecoPropostaService {

    protected $connection;
    protected $environment;
    protected $factoryItFormPrecProp;
    protected $itensPropostaFactory;
    
    public function __construct(EntityManager $entityManager, Sm00001000ItemFormPrecoPropostaFactory $factoryItFormPrecProp, Environment $environment, NewCode $newCode, Sm00001000ItemLancamentoPropostaFactory $itensPropostaFactory) {
        $this->connection            = $entityManager->getConnection();
        $this->factoryItFormPrecProp = $factoryItFormPrecProp;
        $this->environment           = $environment;
        $this->newCode               = $newCode;
        $this->itensPropostaFactory  = $itensPropostaFactory;
    }
    
    /**
     * A função "insereItemFormPrec" prepara o array para inserção dos itens de contrato 
     * e centraliza as funções de inserção com soma ou separadamente.  
     */
    
    public function insereItemFormPrec($servicos, $soma, $proposta, $nrtipoorcamento) {
        $itensInseridos = null;
        
        if ($soma == true) 
            $itensInseridos = $this->insereItFormSoma($servicos, $proposta, $nrtipoorcamento);
        else 
            $itensInseridos = $this->insereItForm($servicos, $proposta, $nrtipoorcamento); 

        return $itensInseridos;
    }

    /**
     * A função "insereItFormSoma" insere os itens de contrato quando escolhida 
     * a opção "Somar os Valores".
     */
    public function insereItFormSoma($itens, $proposta, $nrtipoorcamento) {
        $nrorg    = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        $servicos        = array();
        $valorTotalItens = array();
        $itensInseridos  = array();
       
        $this->connection->beginTransaction();
        try {
            foreach ($itens as &$item) {
                $cdservorca = $item["CDSERVORCA"];
                if (array_key_exists($cdservorca, $servicos)) {
                    $item["NRITEMORCAMENTO"] = $servicos[$cdservorca];
                    
                    if (isset($valorTotalItens[$cdservorca])) 
                        $valorTotalItens[$cdservorca] += $item["VRTOTAL"];
                    
                    $params = array(
                        "VRTOTALITEMORCA" => $item["VRTOTAL"],
                        "CDSERVORCA"      => $cdservorca,
                        "NRSEQORCAMENTO"  => $proposta, 
                        "NRORG"           => $nrorg
                    );
                     
                    $this->factoryItFormPrecProp->somaContFormPreco($params, $item["IDUSAMEDIDA"]);
                } else {
                    $novoCodigo = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
                    $item["NRITEMORCAMENTO"]      = $novoCodigo;
                    $valorTotalItens[$cdservorca] = $item["VRTOTAL"];
                    
                    $params = array(
                        "NRITEMORCAMENTO"    => $novoCodigo, 
                        "NRSEQORCAMENTO"     => $proposta, 
                        "CDSERVORCA"         => $item["CDSERVORCA"],
                        "NMITEMORCAMENTO"    => $item["NMSERVORCA"]." - ".$item["NMELEMENTO"],
                        "QTITEMORCAMENTO"    => $item["QTCALCULO"],
                        "CDTPMEDIDASERV"     => $item["CDTPMEDIDASERV"],
                        "QTMEDIDAORCA"       => is_null($item["QTMEDIDACALC"]) ? 1 : $item["QTMEDIDACALC"],
                        "VRUNITITEMORCA"     => $item["VRUNITARIO"],
                        "VRTOTALITEMORCA"    => $item["VRTOTAL"],
                        "IDMULTMEDVRTOT"     => $item["IDUSAMEDIDA"],
                        "NRORG"              => $nrorg,
                        "CDOPERADOR"         => $operador,
                        "NRTIPOORCAMENTO"    => $nrtipoorcamento,
                        "NRITEMORCAMENTOPAI" => null,
                        "CDTPMEDIDAVARIAVEL" => null
                    );
                    
                    $medida = $this->getMedidaVariavelEscopo($nrtipoorcamento, $item["CDSERVORCA"], $nrorg);
                    if (!empty($medida)) {
                        $params["CDTPMEDIDASERV"]     = $medida["CDTPMEDIDAVARIAVEL"];
                        $params["CDTPMEDIDAVARIAVEL"] = $medida["CDTPMEDIDAVARIAVEL"];
                        $params["IDMULTMEDVRTOT"]     = "S";
                    }
                
                    $this->factoryItFormPrecProp->insereItemContFormPreco($params);
                    
                    if (!empty($medida)) {
                        $params["INSERIR_MEDIDA"] = true;
                    }
                    
                    $this->inserirRelacionamentoFormacaoPrecoProposta($item["NRFORMPRECOSERVICO"], $novoCodigo, $proposta, $nrorg, $operador);
                    
                    $servicos[$item["CDSERVORCA"]] = $novoCodigo;
                    $itensInseridos[] = array_merge($params, array("IDELEMENTO" => $item["IDELEMENTO"]));
                }
            }
            unset($item);
            
            $this->geraRateios($itens, $proposta, true, $valorTotalItens, $nrtipoorcamento);
            $this->connection->commit();
            
            foreach($itensInseridos as &$item) {
                $params = array (
                    "NRITEMORCAMENTO" => $item["NRITEMORCAMENTO"],
                    "NRSEQORCAMENTO"  => $proposta, 
                    "NRORG"           => $nrorg
                );
                
                $totalItem = $this->factoryItFormPrecProp->getValorTotalItem($params);
                $totalItem = $totalItem[0]["VRTOTALITEMORCA"];
                
                $item["VRTOTALITEMORCA"] = $totalItem;
            }
            return $itensInseridos;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    /**
     * A função "insereItForm" insere os itens de contrato quando escolhida 
     * a opção "incluir Separadamente" ou quando não possuem serviços iguais.
     */
    public function insereItForm($itens, $proposta, $nrtipoorcamento) {
        $nrorg          = $this->environment->getNrOrg();
        $operador       = $this->environment->getCdOperador();
        $itensInseridos = array();
        
        $this->connection->beginTransaction();
        try {
            foreach ($itens as &$item) {
                $novoCodigo = $this->newCode->getCodeByTableName("NRITEMORCAMENTO", $nrorg, 1, 10, true);
                $item["NRITEMORCAMENTO"] = $novoCodigo;
                
                $params = array(
                    "NRITEMORCAMENTO"    => $novoCodigo, 
                    "NRSEQORCAMENTO"     => $proposta, 
                    "CDSERVORCA"         => $item["CDSERVORCA"],
                    "NMITEMORCAMENTO"    => $item["NMSERVORCA"]." - ".$item["NMELEMENTO"],
                    "QTITEMORCAMENTO"    => $item["QTCALCULO"],
                    "CDTPMEDIDASERV"     => $item["CDTPMEDIDASERV"],
                    "QTMEDIDAORCA"       => is_null($item["QTMEDIDACALC"]) ? 1 : $item["QTMEDIDACALC"],
                    "VRUNITITEMORCA"     => $item["VRUNITARIO"],
                    "VRTOTALITEMORCA"    => $item["VRTOTAL"],
                    "IDMULTMEDVRTOT"     => $item["IDUSAMEDIDA"],
                    "NRORG"              => $nrorg,
                    "CDOPERADOR"         => $operador,
                    "NRTIPOORCAMENTO"    => $nrtipoorcamento,
                    "NRITEMORCAMENTOPAI" => null,
                    "CDTPMEDIDAVARIAVEL" => null
                );
                
                $medida = $this->getMedidaVariavelEscopo($nrtipoorcamento, $item["CDSERVORCA"], $nrorg);
                if (!empty($medida)) {
                    $params["CDTPMEDIDASERV"]     = $medida["CDTPMEDIDAVARIAVEL"];
                    $params["CDTPMEDIDAVARIAVEL"] = $medida["CDTPMEDIDAVARIAVEL"];
                    $params["IDMULTMEDVRTOT"]     = "S";
                }
                
                $this->factoryItFormPrecProp->insereItemContFormPreco($params);
                
                if (!empty($medida)) {
                    $params["INSERIR_MEDIDA"] = true;
                }
                
                $this->inserirRelacionamentoFormacaoPrecoProposta($item["NRFORMPRECOSERVICO"], $novoCodigo, $proposta, $nrorg, $operador);
                $itensInseridos[] = array_merge($params, array("IDELEMENTO" => $item["IDELEMENTO"]));
            }
            unset($item);
            $this->geraRateios($itens, $proposta, false, null, $nrtipoorcamento);
            $this->connection->commit();
            return $itensInseridos;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function getMedidaVariavelEscopo($nrtipoorcamento, $cdservorca, $nrorg) {
        $params = array(
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "CDSERVORCA"      => $cdservorca,
            "NRORG"           => $nrorg
        );

        $dataset = $this->itensPropostaFactory->getMedidaVariavelEscopo($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    public function inserirRelacionamentoFormacaoPrecoProposta($nrformprecoservico, $nritemorcamento, $nrseqorcamento, $nrorg, $cdoperador) {
        $params = array(
            "NRRELACFORMACAOPROP" => $this->newCode->getCodeByTableName("FCT_RELACFORMACAOPROP", $nrorg, 1, 10, true),
            "NRFORMPRECOSERVICO"  => $nrformprecoservico,
            "NRSEQORCAMENTO"      => $nrseqorcamento,
            "NRITEMORCAMENTO"     => $nritemorcamento,
            "IDATIVO"             => "S",
            "NRORG"               => $nrorg,
            "CDOPERINCLUSAO"      => $cdoperador
        );
        
        $this->factoryItFormPrecProp->inserirRelacionamentoFormacaoPrecoProposta($params);
    }
    
    /**
     *  A função "geraRateios" organiza os arrays para a geração dos arrays e 
     *  dispara a inserção.
     */
    public function geraRateios($itens, $proposta, $soma = false, $valorTotalItens = null, $nrtipoorcamento = null) {
        $nrorg      = $this->environment->getNrOrg();
        $operador   = $this->environment->getCdOperador();
        $geraRateio = $this->getIdGeraRateioParametros($nrtipoorcamento, $nrorg);
        if($geraRateio[0]["IDGERARATEIO"] == 'N') {
            $idrelacestrutura = "PROPOSTA";
        } else {
            $possuiCentroCusto = true;
            $estruturas        = array();
            
            if(count($itens) == 0) {
                $newItens = -1;
            } else {
                $newItens = array();
                foreach($itens as $item) {
                    array_push($newItens, $item["NRITEMORCAMENTO"]);
                    array_push($estruturas, $item["NRESTRUTURAM"]);
                }
                
                $newItens   = implode(",", $newItens);
                $estruturas = implode(",", $estruturas);
                
                $centrocustos = $this->factoryItFormPrecProp->getEstruturasSemCentroCusto($estruturas);
                $possuiCentroCusto = count($centrocustos) > 0;
            }

            $idrelacestrutura = ($possuiCentroCusto) ? "PROPOSTA" : "RATEIO";
            
            if($possuiCentroCusto) {
                $rateiosProposta   = $this->getRateiosItensProposta($proposta, $nrorg, $newItens);
                $possuiItensSemEst = false;
                if(count($rateiosProposta) > 0) {
                    foreach($rateiosProposta as $estrutura)
                        if($estrutura["NRESTRUTURAM"] == null)
                            $idrelacestrutura = "PROPOSTA";
                }
            }
        }
        
        foreach ($itens as $item) {
            $cdservorca       = $item["CDSERVORCA"];
            $estrutura        = $item["NRESTRUTURAM"];
            $vrRateio         = $item["VRTOTAL"];
            $nritemorcamento  = $item["NRITEMORCAMENTO"];
            
            if(!empty($estrutura)) {
                if (!$soma) {
                    $percentRateio = 100;
                } else {
                    $valorTotalItem = $valorTotalItens[$cdservorca];
                    $percentRateio  = $vrRateio / $valorTotalItem * 100;
                }
                
                $novoCodigo = $this->newCode->getCodeByTableName("NRRELACESTRUTPROP", $nrorg, 1, 10, true);
                $params = array(
                    "NRRELACESTRUTPROP" => $novoCodigo,
                    "NRSEQORCAMENTO"    => $proposta,
                    "NRITEMORCAMENTO"   => $nritemorcamento,
                    "NRESTRUTURAM"      => $estrutura,
                    "VRITEMRATEIO"      => $vrRateio,
                    "NRORG"             => $nrorg,
                    "IDATIVO"           => "S",
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $operador,
                    "VRPERCRATEIO"      => $percentRateio,
                    "IDRELACESTRUTURA"  => $idrelacestrutura
                );
                
                $this->factoryItFormPrecProp->insereRateio($params);
            }
        }
        
        $this->atualizarTpEstruturaRateiosInseridos($proposta, $nrorg, $idrelacestrutura);
    }
    
    /**
     * Atualiza o tipo de estrutura dos rateios de uma proposta
    **/
    public function atualizarTpEstruturaRateiosInseridos($proposta, $nrorg, $idrelacestrutura) {
        $params = array(
            "NRSEQORCAMENTO"   => $proposta,
            "IDRELACESTRUTURA" => $idrelacestrutura,
            "NRORG"            => $nrorg
        );
        $this->factoryItFormPrecProp->atualizarTpEstruturaRateiosInseridos($params);
    }
    
   /**
    *  A função "insereRateios" insere os rateios para os itens do contrato
    */
    public function insereRateios($itens, $proposta, $inseridos){
        $nrorg    = $this->environment->getNrOrg();
        $operador = $this->environment->getCdOperador();
        
        $count = count($itens["CDSERVORCA"]);
        // for criado para rodar os itens e gerar os ratieos para cada item
        for ($i = 0; $i < $count; $i++) {
            // Chave para buscar os itens do contrato ja inseriudos de acorco com o serviço
            $key = $itens["CDSERVORCA"][$i];
            
            $nritemorcamento = $inseridos[$key];
            $estrutura       = $itens["NRESTRUTURAM"][$i];
            $vritemrateio    = $itens["VRUNITARIO"][$i];

            if ($estrutura == null){
                throw new \Exception ("Operação Bloquada.<br/>A geração dos rateios não pôde ser realizada pois foi selecionada uma formação de preço sem local de atendimento parametrizado. Favor verificar o cadastro das formações de preço.");
            } else {
                //Para o primeiro índice do For, sempre insere o rateio
                if ($i == 0){
                    $novoCodigo = $this->newCode->getCodeByTableName("FCT_RELACESTRUTURAPROP", $nrorg, 1, 10, true);
                    $params = array(
                        "NRRELACESTRUTPROP" => $novoCodigo, 
                        "NRSEQORCAMENTO"    => $proposta, 
                        "NRITEMORCAMENTO"   => $nritemorcamento, 
                        "NRESTRUTURAM"      => $estrutura,
                        "VRITEMRATEIO"      => $vritemrateio, 
                        "NRORG"             => $nrorg,
                        "CDOPERADOR"        => $operador
                    );
                    
                    $this->factoryItFormPrecProp->insereRateio($params);
                } else {
                    // Valida se, de acordo com o item e a estrutura, já existe um registro criado. 
                    // se existir, atualiza o valor do rateio. Se não existir, insere o rateio.
                    $existe = $this->buscaRateio($proposta, $nritemorcamento, $estrutura);    
                     if ($existe == true){
                         $params = array(
                            "VRITEMRATEIO"      => $vritemrateio, 
                            "NRSEQORCAMENTO"    => $proposta, 
                            "NRITEMORCAMENTO"   => $nritemorcamento, 
                            "NRESTRUTURAM"      => $estrutura,
                            "NRORG"             => $nrorg
                        );
                        
                        $this->factoryItFormPrecProp->atualizaRateio($params);
                    } else {
                        $novoCodigo = $this->newCode->getCodeByTableName("FCT_RELACESTRUTURAPROP", $nrorg, 1, 10, true);
                        $params = array(
                            "NRRELACESTRUTPROP" => $novoCodigo, 
                            "NRSEQORCAMENTO"    => $proposta, 
                            "NRITEMORCAMENTO"   => $nritemorcamento, 
                            "NRESTRUTURAM"      => $estrutura,
                            "VRITEMRATEIO"      => $vritemrateio, 
                            "NRORG"             => $nrorg,
                            "CDOPERADOR"        => $operador
                        );
                        
                        $this->factoryItFormPrecProp->insereRateio($params);
                    }
                }
            }
        }
    }
    
    /**
     *  A função "buscaRateio" verifica se ja existe o rateio para o item e estrutura
     */
    public function buscaRateio($proposta, $nrItem, $estrutura){
        $nrorg = $this->environment->getNrOrg();
        $params = array("NRSEQORCAMENTO"    => $proposta, 
                        "NRITEMORCAMENTO"   => $nrItem, 
                        "NRESTRUTURAM"      => $estrutura, 
                        "NRORG"             => $nrorg
        );
        
        $return = $this->factoryItFormPrecProp->buscaRateio($params);
        return $return[0]["EXISTE"] != 0;
        
    }
    
    /**
     * Insere a nova formação de preço criada pelo usuário 
     * durante a inserção da proposta
    **/
    public function editaItensFormacaoAux($rows, $nrorg, $operador){
        if(is_array($rows)){
            foreach ($rows as $row){
                $iditemcalculo = implode("','", $row["IDITEMCALCULO"]);
                $params = array(
                    "NRORG"                 => $nrorg,
                    "NRFORMPRECOSERVICO"    => $row["NRFORMPRECOSERVICO"],
                    "NRSEQORCAMENTO"        => $row["NRSEQORCAMENTO"],
                    "NRITEMORCAMENTO"       => $row["NRITEMORCAMENTO"],
                );
                
                $this->factoryItFormPrecProp->editaItensFormacaoAux($params, $iditemcalculo);
            }
        }
    }  
 
    /**
     * Executa a procedure que insere a nova formação e altera o 
     * código do pai para o novo inserido
    **/
    public function executarProcedureFctCopiaFormPrecoItProp($nritemorcamento, $nrseqorcamento, $nrformprecoservico, $nrorg, $operador){
        $procedure  = new FctCopiaFormPrecoItprop($this->connection);
        $procedure->executar($nritemorcamento, $nrseqorcamento, $nrformprecoservico, $nrorg, $operador, null, null);
    }
    
    /**
     * Deleta as formações de preço que não foram utilizadas pelo usuário
    **/
    public function deletaAlteracoesInutilizadas($nrseqorcamento, $nritemorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRITEMORCAMENTO" => $nritemorcamento,
            "NRORG"           => $nrorg
        );
        
        $this->factoryItFormPrecProp->deletaAlteracoesInutilizadas($params);
    }
    
    /**
     * Deleta o item de formação de preço inserido quando a rotina de alteração do número 
     * do item na tabela de formção de preço auxiliar não funciona
    **/
    public function deletarItensFormacaoProp($nrseqorcamento, $nritemorcamento, $nrorg){
        $nritemorcamento = implode(",", $nritemorcamento);
        $params = array(
            "NRSEQORCAMENTO"     => $nrseqorcamento,
            "NRORG"              => $nrorg,
        );
        
        $this->factoryItFormPrecProp->deletarItensFormacaoProp($params, $nritemorcamento);
    }

    /**
     * deleta a árvore completa de elementos alterados na formação de 
     * preço auxiliar já inserida na proposta
     **/
    public function deletarArvoreFormPrecoItPro($nrseqorcamento, $nritemorcamento, $nrorg, $nrformprecoservico, $nrcalcformprec){
        $params = array(
            "NRSEQORCAMENTO"     => $nrseqorcamento,
            "NRITEMORCAMENTO"    => $nritemorcamento,
            "NRORG"              => $nrorg,
            "NRCALCFORMPRECSERV" => $nrcalcformprec,
            "NRFORMPRECOSERVICO" => $nrformprecoservico
        );
        
        $this->factoryItFormPrecProp->deletarArvoreFormPrecoItPro($params);
    }
    /**
     * Deleta o relacionamento entre formação e proposta após a exclusão de um item
    **/
    public function deletarRelacFormacaoProp($nrseqorcamento, $nritemorcamento, $nrorg){
        $params = array(
            "NRSEQORCAMENTO"     => $nrseqorcamento,
            "NRITEMORCAMENTO"    => $nritemorcamento,
            "NRORG"              => $nrorg
        );
        $this->factoryItFormPrecProp->deletarRelacFormacaoProp($params);
    }
    
    /**
     * Busca os rateios ligados a proposta
    **/
    public function getRateiosItensProposta($proposta, $nrorg, $itens) {
        $params = array(
            "NRSEQORCAMENTO"     => $proposta,
            "NRORG"              => $nrorg,
        );
        return $this->factoryItFormPrecProp->getRateiosItensProposta($params, $itens);
    }
    
    /**
     * Busca os dados do parametro de faturamento do escopo ligado ao item
    **/
    public function getIdGeraRateioParametros($nrtipoorcamento, $nrorg) {
        if(is_array($nrtipoorcamento)){
            $nrtipoorcamento = implode(",", $nrtipoorcamento);
        }       
        return $this->factoryItFormPrecProp->getIdGeraRateioParametros($nrtipoorcamento, $nrorg);
    }
    /**
     * retorna as estruturas ligadas ao rateio 
    **/
    public function validarDadosRateioProposta($nrseqorcamento, $nrorg, $newEstruturas, $nrtipoorcamento) {
        $paramsFaturmento = $this->getIdGeraRateioParametros($nrtipoorcamento, $nrorg);
        if(count($paramsFaturmento) > 0 && $paramsFaturmento[0]["IDGERARATEIO"] == 'S') {
            $estruturasAtuais = implode(",", $newEstruturas);
            $params = array(
                "NRSEQORCAMENTO" => $nrseqorcamento,
                "NRORG"          => $nrorg
            );
            
            if($estruturasAtuais) {
                $centrocustos = $this->factoryItFormPrecProp->getEstruturasSemCentroCusto($estruturasAtuais);
                if(count($centrocustos) > 0)
                    return "SEM_CENTROCUST";
            }
            
            $rateiosProposta = $this->factoryItFormPrecProp->getTotalEstrututasItensProposta($params);
            if($rateiosProposta[0]["COUNTITENSRATEIO"] > 0) {
                if(count($newEstruturas) > 0 && in_array(null, $newEstruturas))
                    return "RATEIO_SERA_DELETADO";
            }
        }
        
        return null;
    }
}
?>